/**
 * This function vertically positions a span containing a reading, either at the
 * same height as the corresponding lemma, or at a different height when it would
 * overlap other readings.
 *
 * @function
 * @param {object} rdg A span containing a reading.
 */
const positionRdg = function positionRdg(rdg) {
  const openRdgs = document.getElementsByClassName('rdg-digital--show');
  const lemmaId = rdg.getAttribute('target-lem').replace('#', '');
  const lemma = document.getElementById(lemmaId);
  const rdgHeight = rdg.offsetHeight < 44 ? 44 : rdg.offsetHeight;
  let rdgTop;
  if (lemma.offsetTop < 65) {
    rdgTop = lemma.parentElement.offsetTop;
  } else {
    rdgTop = lemma.offsetTop;
  }
  let rdgBottom = rdgTop + rdgHeight;
  // sort open readings by position, as this might not be the same order as in the DOM
  const rdgsArray = [...openRdgs].sort((a, b) => a.offsetTop - b.offsetTop);

  rdgsArray.forEach((openRdg) => {
    if (openRdg !== rdg) {
      const openRdgTop = openRdg.offsetTop;
      const openRdgBottom = openRdgTop + openRdg.offsetHeight;
      if (rdgTop >= openRdgBottom + 5 || rdgBottom <= openRdgTop - 5) { return; }
      if (
        (rdgTop >= openRdgTop - 5 && rdgTop <= openRdgBottom + 5)
        || (rdgBottom >= openRdgTop - 5 && rdgBottom <= openRdgBottom + 5)
      ) {
        rdgTop = openRdgBottom + 5;
        rdgBottom = rdgTop + rdgHeight;
      }
    }
  });
  rdg.style.top = `${rdgTop}px`;
};

/**
 * This function opens a reading, positions it, highlights the corresponding lemma.
 * This function is called when one of the
 * variant buttons is clicked.
 *
 * @function
 * @param {object} rdg A span containing a reading.
 */
const openRdg = function openRdg(rdg) {
  let lemmaId = rdg.getAttribute('target-lem').replace('#', '');
  let lemma = document.getElementById(lemmaId);

  lemma.classList.add('lemma--open');
  rdg.classList.add('rdg-digital--show');
  positionRdg(rdg);
};

/**
 * This function closes a reading when the reading, its corresponding lemma or
 * the variant button is clicked.
 *
 * @function
 * @param {object} rdg A span containing a reading.
 */
const closeRdg = function closeRdg(rdg) {
  event.stopPropagation();
  let lemmaId = rdg.getAttribute('target-lem').replace('#', '');
  let lemma = document.getElementById(lemmaId);

  /** do not remove highlighting from corresponding lemma if there is another
     * open reading corresponding to that lemma.
     */
  document.querySelectorAll(`.rdg-digital--show[target-lem='#${lemmaId}']`).length > 1 ? '' : lemma.classList.remove('lemma--open');
  rdg.classList.remove('rdg-digital--show');
  rdg.style.top = '';
};

/**
 * This function handles the highlighting of a lemma when clicked and calls the
 * display of any relevant readings.
 *
 * @function
 * @param {object} lem A span containing a lemma.
 */
const openLemma = function openLemma(lem) {
  let id = lem.id;
  let relevantRdgs = document.querySelectorAll(`.rdg-digital[target-lem='#${id}']`);

  lem.classList.add('lemma--open');
  for (rdg of relevantRdgs) {
    positionRdg(rdg);
    rdg.classList.add('rdg-digital--show');
  };
};

/**
 * This function closes a lemma and all its corresponding readings.
 *
 * @function
 * @param {object} lem A span containing a lemma.
 */
const closeLemma = function closeLemma(lem) {
  let id = lem.id;
  let relevantRdgs = document.querySelectorAll(`.rdg-digital[target-lem='#${id}']`);

  lem.classList.remove('lemma--open');
  for (rdg of relevantRdgs) {
    closeRdg(rdg);
  };
};

/**
 * This is a wrapper function, checking whether a clicked lemma is open or closed.
 *
 * @function
 */
const toggleLemma = function toggleLemma() {
  this.classList.contains('lemma--open') ? closeLemma(this) : openLemma(this);
};


/**
 * This function highlights a lemma's correspoding reading or
 * readings on mouseover.
 *
 * @function
 */
const mouseOverLem = function mouseOverLem() {
  let id = this.id;
  let rdg = document.querySelector(`[target-lem='#${id}']`);

  rdg.classList.toggle('highlight');
};

/**
 * This function highlights a reading on mouseover, as well as its corresponding
 * lemma.
 *
 * @function
 */
const mouseOverRdg = function mouseOverRdg() {
  let lemId = this.getAttribute('target-lem').replace('#', '');
  let lem = document.getElementById(lemId);

  this.classList.toggle('highlight');
  lem.classList.toggle('highlight');
};

/**
 * This function sets the position
 * of any opened readings when the screen is
 * resized.
 *
 * @function
 */
const onScreenResize = function onScreenResize() {
  let openRdgs = document.querySelectorAll('.rdg-digital--show');

  for (rdg of openRdgs) {
    let lemmaId = rdg.getAttribute('target-lem').replace('#', '');
    let lemma = document.getElementById(lemmaId);
    positionRdg(rdg);
  };
};

/**
 * Get all lemmata and readings.
 */
const lems = document.getElementsByClassName('lemma');
const rdgs = document.getElementsByClassName('rdg-digital');

window.addEventListener('resize', onScreenResize);

for (let i = 0; i < lems.length;) {
  lems[i].addEventListener('click', toggleLemma, false);
  lems[i].addEventListener('mouseenter', mouseOverLem, false);
  lems[i].addEventListener('mouseleave', mouseOverLem, false);
  i += 1;
};

for (let i = 0; i < rdgs.length;) {
  rdgs[i].addEventListener('click', function(){closeRdg(this);}, false);
  rdgs[i].addEventListener('mouseenter', mouseOverRdg, false);
  rdgs[i].addEventListener('mouseleave', mouseOverRdg, false);
  i += 1;
};

// override of anchor behavior for anchors whose target is in a reading
const anchors = document.querySelectorAll('.link--text--digital');
anchors.forEach((anchor) => {
  anchor.addEventListener('click', (e) => {
    e.preventDefault();
    const href = anchor.getAttribute('href');
    const target = document.getElementById(href.replace('#', ''));
    const rdg = target.closest('.rdg-digital');
    openRdg(rdg);
    target.scrollIntoView({ behavior: 'smooth' });
  });
});
